"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const eventemitter3_1 = require("eventemitter3");
const { logDecoder, } = require("ethjs-abi");
const abi_1 = require("./abi");
const TxReceiptPromise_1 = require("./TxReceiptPromise");
class Contract {
    constructor(rpc, info) {
        this.rpc = rpc;
        this.info = info;
        this.callMethodsMap = {};
        this.sendMethodsMap = {};
        for (const methodABI of info.abi) {
            const name = methodABI.name;
            // Allow sendToContract only for non-constant methods
            if (!methodABI.constant) {
                this.sendMethodsMap[name] = methodABI;
            }
            this.callMethodsMap[name] = methodABI;
        }
        this.address = info.address;
    }
    encodeParams(method, args = []) {
        const methodABI = this.callMethodsMap[method];
        if (methodABI == null) {
            throw new Error(`Unknown method to call: ${method}`);
        }
        return abi_1.encodeInputs(methodABI, args);
    }
    /**
     * Call a contract method using ABI encoding, and return the RPC result as is. This
     * does not create a transaction. It is useful for gas estimation or getting results from
     * read-only methods.
     *
     * @param method name of contract method to call
     * @param args arguments
     */
    async rawCall(method, args = [], opts = {}) {
        // TODO opts: sender address
        // need to strip the leading "0x"
        const calldata = this.encodeParams(method, args);
        // TODO decode?
        return this.rpc.callContract(Object.assign({ address: this.address, datahex: calldata, senderAddress: opts.senderAddress || this.info.sender }, opts));
    }
    async call(method, args = [], opts = {}) {
        // TODO support the named return values mechanism for decodeParams
        const r = await this.rawCall(method, args, opts);
        const exception = r.executionResult.excepted;
        if (exception !== "None") {
            throw new Error(`Call exception: ${exception}`);
        }
        const output = r.executionResult.output;
        let decodedOutputs = [];
        if (output !== "") {
            const methodABI = this.callMethodsMap[method];
            decodedOutputs = abi_1.decodeOutputs(methodABI, output);
        }
        return Object.assign(r, {
            outputs: decodedOutputs,
        });
    }
    /**
     * Create a transaction that calls a method using ABI encoding, and return the RPC result as is.
     * A transaction will require network consensus to confirm, and costs you gas.
     *
     * @param method name of contract method to call
     * @param args arguments
     */
    async rawSend(method, args, opts = {}) {
        // TODO opts: gas limit, gas price, sender address
        const methodABI = this.sendMethodsMap[method];
        if (methodABI == null) {
            throw new Error(`Unknown method to send: ${method}`);
        }
        const calldata = abi_1.encodeInputs(methodABI, args);
        return this.rpc.sendToContract(Object.assign({ address: this.address, datahex: calldata, senderAddress: opts.senderAddress || this.info.sender }, opts));
    }
    async confirm(tx, confirm, onConfirm) {
        const txrp = new TxReceiptPromise_1.TxReceiptPromise(this.rpc, tx.txid);
        if (onConfirm) {
            txrp.onConfirm((tx2, receipt2) => {
                const sendTxReceipt = this._makeSendTxReceipt(receipt2);
                onConfirm(tx2, sendTxReceipt);
            });
        }
        const receipt = await txrp.confirm(confirm);
        return this._makeSendTxReceipt(receipt);
    }
    async send(method, args, opts = {}) {
        const methodABI = this.sendMethodsMap[method];
        if (methodABI == null) {
            throw new Error(`Unknown method to send: ${method}`);
        }
        const calldata = abi_1.encodeInputs(methodABI, args);
        const sent = await this.rpc.sendToContract(Object.assign({ datahex: calldata, address: this.address, senderAddress: opts.senderAddress || this.info.sender }, opts));
        const txid = sent.txid;
        const txinfo = await this.rpc.getTransaction({ txid });
        const sendTx = Object.assign({}, txinfo, { method, confirm: (n, handler) => {
                return this.confirm(sendTx, n, handler);
            } });
        return sendTx;
    }
    async logs(req = {}) {
        const filter = req.filter || {};
        if (!filter.addresses) {
            filter.addresses = [this.address];
        }
        const result = await this.rpc.waitforlogs(Object.assign({}, req, { filter }));
        const entries = result.entries.map((entry) => {
            const parsedLog = this.logDecoder.decode(entry);
            return Object.assign({}, entry, { event: parsedLog });
        });
        return Object.assign({}, result, { entries });
    }
    onLog(fn, opts = {}) {
        let nextblock = opts.from || "latest";
        const loop = async () => {
            while (true) {
                const result = await this.logs(Object.assign({}, opts, { from: nextblock }));
                for (const entry of result.entries) {
                    fn(entry);
                }
                nextblock = result.nextblock;
            }
        };
        loop();
    }
    /**
     * events API for getting logs
     *
     * logs = token.logEmitter({ minconf: 1 })
     *
     * logs.on("Mint", (logEntry: IContractLogEntry) => {
     *   // ...
     * })
     *
     * logs.on("Transfer", (logEntry: IContractLogEntry) => {
     *   // ...
     * })
     *
     * logs.on("?", () => {
     *   // catch all for unparsed events not defined in ABI
     * })
     *
     */
    logEmitter(opts = {}) {
        const emitter = new eventemitter3_1.EventEmitter();
        this.onLog((entry) => {
            const key = (entry.event && entry.event.type) || "?";
            emitter.emit(key, entry);
        }, opts);
        return emitter;
    }
    get logDecoder() {
        if (this._logDecoder) {
            return this._logDecoder;
        }
        this._logDecoder = new abi_1.ContractLogDecoder(this.info.abi);
        return this._logDecoder;
    }
    _makeSendTxReceipt(receipt) {
        // https://stackoverflow.com/a/34710102
        // ...receiptNoLog will be a copy of receipt, without the `log` property
        const { log: rawlogs } = receipt, receiptNoLog = __rest(receipt, ["log"]);
        const logs = abi_1.decodeLogs(this.info.abi, rawlogs);
        return Object.assign({}, receiptNoLog, { logs,
            rawlogs });
    }
}
exports.Contract = Contract;
