"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const { decodeParams, encodeMethod, logDecoder, } = require("ethjs-abi");
function ensureHex0x(hexstr) {
    if (hexstr[0] === "0" && hexstr[1] === "x") {
        return hexstr;
    }
    return "0x" + hexstr;
}
function encodeInputs(method, args = []) {
    args = args.map((arg, i) => {
        const input = method.inputs[i];
        if (input.type === "address") {
            return ensureHex0x(arg);
        }
        return arg;
    });
    // slice to strip the leading "0x"
    const calldata = encodeMethod(method, args).slice(2);
    return calldata;
}
exports.encodeInputs = encodeInputs;
function decodeOutputs(method, outputData) {
    const types = method.outputs.map((output) => output.type);
    const result = decodeParams(types, "0x" + outputData);
    // NB: Result is an "array-like" object like arguments.
    // But apparently the following clone technique doesn't work.
    // return Array.prototype.slice.call(result)
    // Convert result to normal array...
    const values = [];
    for (let i = 0; i < types.length; i++) {
        values[i] = result[i];
    }
    return values;
}
exports.decodeOutputs = decodeOutputs;
class ContractLogDecoder {
    constructor(abi) {
        this.abi = abi;
        this._decoder = logDecoder(abi);
    }
    decode(rawlog) {
        const result = this._decoder([{
                data: ensureHex0x(rawlog.data),
                topics: rawlog.topics.map(ensureHex0x),
            }]);
        if (result.length === 0) {
            return null;
        }
        const log = result[0];
        const type = log._eventName;
        const logABI = this.abi.find((method) => method.name === type);
        if (!logABI) {
            throw new Error(`Cannot find ABI for event type: ${type}`);
        }
        const decodedLog = {
            type,
        };
        // logABI.inputs.forEach(())
        for (const input of logABI.inputs) {
            decodedLog[input.name] = log[input.name];
        }
        return decodedLog;
    }
}
exports.ContractLogDecoder = ContractLogDecoder;
function decodeLogs(methods, logs) {
    const decoder = logDecoder(methods);
    // Add the 0x prefix to all hex strings, else abi parsing would fail
    const rawlogs = logs.map((log) => {
        // console.log("rawlog", log)
        return {
            // address: ensureHex0x(log.address),
            data: ensureHex0x(log.data),
            topics: log.topics.map(ensureHex0x),
        };
    });
    const parsedLogs = decoder(rawlogs);
    // Remove the "array-like" behaviour. Just return a map of event parameters.
    return parsedLogs.map((log) => {
        const type = log._eventName;
        const logABI = methods.find((method) => method.name === type);
        if (!logABI) {
            throw new Error(`Cannot find ABI for event type: ${type}`);
        }
        const decodedLog = {
            type,
        };
        // logABI.inputs.forEach(())
        for (const input of logABI.inputs) {
            decodedLog[input.name] = log[input.name];
        }
        return decodedLog;
    });
}
exports.decodeLogs = decodeLogs;
